/**
 * @(#) Patron.java 1.0 2001/10/22
 * 
 * Copyright by Foo Inc. 3000 N. Virginia Street, Reno NV 89557
 * All rights reserved
 *
 * 
 */

package edu.unr.cs.LMS;

import java.util.*;
//import edu.unr.cs.LMS.util.*;
//import edu.unr.cs.LMS.FormLetter.*;
//import edu.unr.cs.LMS.DataBaseAccess.*;

/**
 * The patron class encapsulates the data and behavior of LMS patrons.
 * Students, Faculty, and Staff are types of patrons and inherit from 
 * this class.
 * 
 * @version 1.0 10/22/01
 * @author Sushil J. Louis
 */


public class Patron {
    // class variables

    /**
     * Keeps track of next patronID to be assigned
     */
    private static long nextPatronID = 0;

    // instance variables
    /**
     * Name of patron
     */
    private String name;

    /**
     * This patron object's id
     */
    private long patronID;

    /**
     * This patron's home Phone number
     */
    private long homePhone;

    /**
     * The membership start date for this patron
     */
    private Date membershipDate;

    /**
     * Membership expiration date
     */
    private Date expirationDate;

    /**
     * Reference to list of resources checked out by this patron
     */
    private List resourceList;

    /**
     * Patron's home address
     */
    private Address homeAddress;

    // Constructors

    /**
     * Constructs a patron object.
     * Constructs an address object from parameters and initializes class 
     * variables. 
     * @param name Patron's name as a String in Last, First, MI format
     * @param homePhone Patron's home phone number (3 + 7)
     * @param start The Patron's membership start date as a Date object
     * @param end  The Patron's membership end date as a Date object
     * @param street A string specifying patron's home street
     * @param city A string specifying patron's city
     * @param state A string specifying patron's state
     * @param zip A long specifying patron's zip code
     */

    public Patron(String name, long homePhone, Date start, Date end, 
		  String street, String city, String state, long zip) {

	this.name = name;
	this.homePhone = homePhone;
	this.membershipDate = start;
	this.expirationDate = end;
	this.homeAddress = new Address(street, city, state, zip);
	this.patronID = getNextPatronID();
	resourceList = new ArrayList();
	LMSUtil.log("Constructed Patron: " + this.patronID);
    }
    
    /**
     * Returns a new patron ID. Patron ID's are generated by incrementing the
     * class variable nextPatronID.
     */
    private static long getNextPatronID() {
	LMSUtil.log("Generating next patron ID: " + nextPatronID);
	return nextPatronID++;
    }
    

    /**
     * Returns true if the patron's membership has not expired. Checked 
     * by comparing the input parameter, endDate against the membership 
     * expiration date expirationDate. returns true if
     * expirationDate < endDate else false.
     *
     * @param endDate The date (today's) against which to check membership
     * expiration. This is of type java.util.Date.
     * @return Returns true if expiration date is after endDate. See 
     * @link{Date} for the exact semantics of Date.after
     */
    public boolean validatePatron(Date endDate) {
	LMSUtil.log("Validating Patron: " + patronID + " membership expires today");
	Date today = new Date();
	return expirationDate.after(endDate);
    }
    

    /**
     * Skeleton for checkout. Currently, adds a resourceID object to the 
     * resource List. 
     */
    public boolean checkout(Resource resourceID) {
	LMSUtil.log("Checking out resource " + resourceID);
	//add a resource reference to resourceList
	resourceList.add(resourceID);
	return true;
    }

    //...more methods...

    /**
     * Accessor for patronID, private long identifier that uniquely specifies 
     * a patron.
     */
    public long getPatronID() {
	return patronID;
    }

}
